@@ -22,7 +22,7 @@
 #' \code{cortype = 2} denotes the exponential structure,
 #' where the covariance matrix has \eqn{(i,j)} entry equals \eqn{rho^{|i-j|}}.
 #' code{cortype = 3} denotes the constant structure,
-#' where the non-diagnoal entries of covariance
+#' where the non-diagonal entries of covariance
 #' matrix are \eqn{rho} and diagonal entries are 1.
 #' @param snr A numerical value controlling the signal-to-noise ratio (SNR). The SNR is defined as
 #' as the variance of \eqn{x\beta} divided
@@ -118,7 +118,11 @@ generate.data <- function(n,
                           p,
                           support.size = NULL,
                           rho = 0,
-                          family = c("gaussian", "binomial", "poisson", "cox", "mgaussian", "multinomial"),
+                          family = c(
+                            "gaussian", "binomial", "poisson",
+                            "cox", "mgaussian", "multinomial",
+                            "gamma"
+                          ),
                           beta = NULL,
                           cortype = 1,
                           snr = 10,
@@ -286,8 +290,8 @@ generate.data <- function(n,
     colnames(y) <- paste0("y", 1:y_dim)
   }
   if (family == "multinomial") {
-    m <- 5 * sqrt(2 * log(p) / n)
-    M <- 100 * m
+    m <- 2.5 * sqrt(2 * log(p) / n)
+    M <- 50 * m
     if (is.null(input_beta)) {
       beta[nonzero, ] <- matrix(stats::runif(support.size * y_dim, m, M),
         ncol = y_dim
@@ -305,9 +309,26 @@ generate.data <- function(n,
     prob_y <- exp(prob_y)
     prob_y <- sweep(prob_y, MARGIN = 1, STATS = rowSums(prob_y), FUN = "/")
     y <- apply(prob_y, 1, function(x) {
-      sample(0:2, size = 1, prob = x)
+      sample(0:(length(x) - 1), size = 1, prob = x)
     })
   }
+  if (family == "gamma") {
+    m <- 5 * sqrt(2 * log(p) / n)
+    if (is.null(input_beta)) {
+      # this is the true value of beta
+      beta[nonzero] <- stats::runif(support.size, m, 100 * m)
+    } else {
+      beta <- input_beta
+    }
+    # add noise
+    sigma <- sqrt((t(beta) %*% Sigma %*% beta) / snr)
+    eta <- x %*% beta + stats::rnorm(n, 0, sigma)
+    # set coef_0 as + abs(min(eta)) + 1
+    eta <- eta + abs(min(eta)) + 10
+    # set the shape para of gamma uniformly in [0.1,100.1]
+    shape_para <- 100 * runif(n) + 0.1
+    y <- stats::rgamma(n, shape = shape_para, rate = shape_para * eta)
+  }
   set.seed(NULL)
 
   colnames(x) <- paste0("x", 1:p)
@@ -370,9 +391,10 @@ abess_model_matrix <- function(object, data = environment(object),
     }
   }
   ############################################################
+  y_name <- strsplit(deparse(t), split = " ~ ")[[1]][1]
   if (length(data)) {
     namD <- names(data)
-
+    namD <- setdiff(namD, y_name)
     for (i in namD) {
       if (is.character(data[[i]])) {
         stop("Some columns in data are character! You may convert these columns to a dummy variable via model.matrix function or discard them.")
@@ -384,6 +406,48 @@ abess_model_matrix <- function(object, data = environment(object),
   data
 }
 
+map_tunetype2numeric <- function(tune.type) {
+  ic_type <- switch(tune.type,
+    "aic" = 1,
+    "bic" = 2,
+    "gic" = 3,
+    "ebic" = 4,
+    "cv" = 1
+  )
+  ic_type
+}
+
+check_foldid <- function(foldid, nobs) {
+  stopifnot(is.vector(foldid))
+  stopifnot(is.numeric(foldid))
+  stopifnot(length(foldid) == nobs)
+  check_integer_warning(
+    foldid,
+    "nfolds should be an integer value. It is coerced to be as.integer(foldid). "
+  )
+  foldid <- as.integer(foldid)
+  cv_fold_id <- foldid
+  cv_fold_id
+}
+
+check_nfold <- function(nfolds) {
+  stopifnot(is.numeric(nfolds) & nfolds >= 2)
+  check_integer_warning(
+    nfolds,
+    "nfolds should be an integer value. It is coerced to be as.integer(nfolds). "
+  )
+  nfolds <- as.integer(nfolds)
+  nfolds
+}
+
+map_dgCMatrix2entry <- function(x) {
+  x <- summary(x)
+  x[, 1:2] <- x[, 1:2] - 1
+  x <- as.matrix(x)
+  x <- x[, c(3, 1, 2)]
+  x
+}
+
 MULTIVARIATE_RESPONSE <- c("mgaussian", "multinomial")
 
 .onUnload <- function(libpath) {
