@@ -46,7 +46,6 @@
 #include <Eigen/Eigen>
 #endif
 
-#include "Data.h"
 #include "utilities.h"
 #include <iostream>
 #include <cfloat>
@@ -97,73 +96,41 @@ public:
   Eigen::VectorXi A_out; /* final active set. */
   Eigen::VectorXi I_out; /* final active set. */
 
-  Eigen::Matrix<Eigen::MatrixXd, -1, -1> PhiG;    /* PhiG for linear model. */
-  Eigen::Matrix<Eigen::MatrixXd, -1, -1> invPhiG; /* invPhiG for linear model. */
-  Eigen::Matrix<T4, -1, -1> group_XTX;            /* XTX. */
   bool lambda_change;                             /* lambda_change or not. */
 
   Eigen::VectorXi always_select;    /* always select variable. */
   double tau;                       /* algorithm stop threshold */
   int primary_model_fit_max_iter;   /* The maximal number of iteration for primaty model fit*/
   double primary_model_fit_epsilon; /* The epsilon (threshold) of iteration for primaty model fit*/
-  bool approximate_Newton;          /* use approximate Newton method or not. */
 
   T2 beta_warmstart;  /*warmstart beta.*/
   T3 coef0_warmstart; /*warmstart intercept.*/
 
-  Eigen::VectorXi status;
-
-  Eigen::MatrixXd cox_hessian; /* hessian matrix for cox model. */
-  Eigen::VectorXd cox_g;       /* score function for cox model. */
-
-  bool covariance_update;              /* use covairance update mathod or not. */
-  Eigen::VectorXd **covariance = NULL; /* covairance matrix. */
-  bool *covariance_update_flag = NULL; /* each variable have updated in covairance matirx. */
-  T1 XTy;                              /*X.transpose() * y */
-  T1 XTone;                            /* X.transpose() * Eigen::MatrixXd::one() */
-
   double effective_number; /* effective number of parameter. */
-
   int splicing_type;     /* exchange number update mathod. */
-  Eigen::MatrixXd Sigma; /* covariance matrix for pca. */
-
   int sub_search; /* size of sub_searching in splicing */
   int U_size;
 
-  int pca_n = -1;
-
-  T1 XTy_U;
-  T1 XTone_U;
-  Eigen::Matrix<Eigen::MatrixXd, -1, -1> PhiG_U;
-  Eigen::Matrix<Eigen::MatrixXd, -1, -1> invPhiG_U;
-
   Algorithm() = default;
 
   virtual ~Algorithm(){};
 
-  Algorithm(int algorithm_type, int model_type, int max_iter = 100, int primary_model_fit_max_iter = 10, double primary_model_fit_epsilon = 1e-8, bool warm_start = true, int exchange_num = 5, bool approximate_Newton = false, Eigen::VectorXi always_select = Eigen::VectorXi::Zero(0), bool covariance_update = false, int splicing_type = 0, int sub_search = 0)
+  Algorithm(int algorithm_type, int model_type, int max_iter = 100, int primary_model_fit_max_iter = 10, double primary_model_fit_epsilon = 1e-8, bool warm_start = true, int exchange_num = 5, Eigen::VectorXi always_select = Eigen::VectorXi::Zero(0), int splicing_type = 0, int sub_search = 0)
   {
     this->max_iter = max_iter;
     this->model_type = model_type;
     // this->coef0_init = 0.0;
     this->warm_start = warm_start;
     this->exchange_num = exchange_num;
-    this->approximate_Newton = approximate_Newton;
     this->always_select = always_select;
     this->algorithm_type = algorithm_type;
     this->primary_model_fit_max_iter = primary_model_fit_max_iter;
     this->primary_model_fit_epsilon = primary_model_fit_epsilon;
 
-    this->covariance_update = covariance_update;
-
     this->splicing_type = splicing_type;
     this->sub_search = sub_search;
   };
 
-  void update_PhiG(Eigen::Matrix<Eigen::MatrixXd, -1, -1> &PhiG) { this->PhiG = PhiG; }
-
-  void update_invPhiG(Eigen::Matrix<Eigen::MatrixXd, -1, -1> &invPhiG) { this->invPhiG = invPhiG; }
-
   void set_warm_start(bool warm_start) { this->warm_start = warm_start; }
 
   void update_beta_init(T2 &beta_init) { this->beta_init = beta_init; }
@@ -192,9 +159,7 @@ public:
 
   void update_exchange_num(int exchange_num) { this->exchange_num = exchange_num; }
 
-  void update_group_XTX(Eigen::Matrix<T4, -1, -1> &group_XTX) { this->group_XTX = group_XTX; }
-
-  void update_tau(int train_n, int N)
+  virtual void update_tau(int train_n, int N)
   {
     if (train_n == 1)
     {
@@ -226,33 +191,25 @@ public:
 
   Eigen::VectorXd get_bd() { return this->bd; }
 
-  int get_l() { return this->l; }
+  virtual int get_beta_size(int n, int p) { return p; }
 
-  void fit(T4 &train_x, T1 &train_y, Eigen::VectorXd &train_weight, Eigen::VectorXi &g_index, Eigen::VectorXi &g_size, int train_n, int p, int N, Eigen::VectorXi &status, Eigen::MatrixXd sigma)
+  void fit(T4 &train_x, T1 &train_y, Eigen::VectorXd &train_weight, Eigen::VectorXi &g_index, Eigen::VectorXi &g_size, int train_n, int p, int N)
   {
-
     int T0 = this->sparsity_level;
-    // this->status = status;
-    this->cox_g = Eigen::VectorXd::Zero(0);
-
-    this->update_tau(train_n, N);
-
     this->x = &train_x;
     this->y = &train_y;
     this->beta = this->beta_init;
     this->coef0 = this->coef0_init;
     this->bd = this->bd_init;
-
     if (this->sub_search == 0 || this->sparsity_level + this->sub_search > N)
       this->U_size = N;
     else
       this->U_size = this->sparsity_level + this->sub_search;
 
-    if (this->model_type == 7)
-    {
-      this->Sigma = sigma;
-    }
+    // specific init 
+    this->inital_setting(train_x, train_y, train_weight, g_index, g_size, N);
 
+    // no need to splicing?
     if (N == T0)
     {
       this->A_out = Eigen::VectorXi::LinSpaced(N, 0, N - 1);
@@ -263,34 +220,19 @@ public:
       //   this->beta = beta_old;
       //   this->coef0 = coef0_old;
       // }
-      this->train_loss = neg_loglik_loss(train_x, train_y, train_weight, this->beta, this->coef0, this->A_out, g_index, g_size);
-      this->effective_number = effective_number_of_parameter(train_x, train_x, train_y, train_weight, this->beta, this->beta, this->coef0);
+      this->train_loss = this->loss_function(train_x, train_y, train_weight, this->beta, this->coef0, this->A_out, g_index, g_size, this->lambda_level);
+      this->effective_number = this->effective_number_of_parameter(train_x, train_x, train_y, train_weight, this->beta, this->beta, this->coef0);
       return;
     }
 
-    if (this->model_type == 1 || this->model_type == 5)
-    {
-      // this->covariance = Eigen::MatrixXd::Zero(train_x.cols(), train_x.cols());
-      if ((this->algorithm_type == 6 && this->PhiG.rows() == 0) || this->lambda_change)
-      {
-        this->PhiG = Phi(train_x, g_index, g_size, train_n, p, N, this->lambda_level, this->group_XTX);
-        this->invPhiG = invPhi(PhiG, N);
-        this->PhiG_U.resize(N, 1);
-        this->invPhiG_U.resize(N, 1);
-      }
-    }
-
     // input: this->beta_init, this->coef0_init, this->A_init, this->I_init
     // for splicing get A;for the others 0;
 
-    Eigen::VectorXi A = inital_screening(train_x, train_y, this->beta, this->coef0, this->A_init, this->I_init, this->bd, train_weight, g_index, g_size, N);
-
+    Eigen::VectorXi A = this->inital_screening(train_x, train_y, this->beta, this->coef0, this->A_init, this->I_init, this->bd, train_weight, g_index, g_size, N);
     Eigen::VectorXi I = Ac(A, N);
-    // Eigen::MatrixXi A_list(T0, max_iter + 2);
-    // A_list.col(0) = A;
 
-    Eigen::VectorXi A_ind = find_ind(A, g_index, g_size, p, N);
-    T4 X_A = X_seg(train_x, train_n, A_ind);
+    Eigen::VectorXi A_ind = find_ind(A, g_index, g_size, (this->beta).rows(), N);
+    T4 X_A = X_seg(train_x, train_n, A_ind, this->model_type);
     T2 beta_A;
     slice(this->beta, A_ind, beta_A);
 
@@ -303,39 +245,42 @@ public:
     //   this->coef0 = coef0_old;
     // }else{
     slice_restore(beta_A, A_ind, this->beta);
-    this->train_loss = neg_loglik_loss(X_A, train_y, train_weight, beta_A, this->coef0, A, g_index, g_size);
+    this->train_loss = this->loss_function(X_A, train_y, train_weight, beta_A, this->coef0, A, g_index, g_size, this->lambda_level);
     // }
 
     // for (int i=0;i<A.size();i++) cout<<A(i)<<" ";cout<<endl<<"init loss = "<<this->train_loss<<endl;
     // }
 
+    // start splicing
     this->beta_warmstart = this->beta;
     this->coef0_warmstart = this->coef0;
 
     int always_select_size = this->always_select.size();
     int C_max = min(min(T0 - always_select_size, this->U_size - T0 - always_select_size), this->exchange_num);
 
+    this->update_tau(train_n, N);
     this->get_A(train_x, train_y, A, I, C_max, this->beta, this->coef0, this->bd, T0, train_weight, g_index, g_size, N, this->tau, this->train_loss);
 
-    // final fit
+    if (this->model_type < 7){
+      // final fit
+      A_ind = find_ind(A, g_index, g_size, (this->beta).rows(), N);
+      X_A = X_seg(train_x, train_n, A_ind, this->model_type);
+      slice(this->beta, A_ind, beta_A);
+
+      this->primary_model_fit_max_iter += 20;
+      // coef0_old = this->coef0;
+      success = this->primary_model_fit(X_A, train_y, train_weight, beta_A, this->coef0, DBL_MAX, A, g_index, g_size);
+      // if (!success){
+      //   this->coef0 = coef0_old;
+      // }else{
+      slice_restore(beta_A, A_ind, this->beta);
+      this->train_loss = this->loss_function(X_A, train_y, train_weight, beta_A, this->coef0, A, g_index, g_size, this->lambda_level);
+      // }
+      this->primary_model_fit_max_iter -= 20;
+    }
+
     this->A_out = A;
-
-    A_ind = find_ind(A, g_index, g_size, p, N);
-    X_A = X_seg(train_x, train_n, A_ind);
-    slice(this->beta, A_ind, beta_A);
-
-    this->primary_model_fit_max_iter += 20;
-    // coef0_old = this->coef0;
-    success = this->primary_model_fit(X_A, train_y, train_weight, beta_A, this->coef0, DBL_MAX, A, g_index, g_size);
-    // if (!success){
-    //   this->coef0 = coef0_old;
-    // }else{
-    slice_restore(beta_A, A_ind, this->beta);
-    this->train_loss = neg_loglik_loss(X_A, train_y, train_weight, beta_A, this->coef0, A, g_index, g_size);
-    // }
-    this->primary_model_fit_max_iter -= 20;
-
-    this->effective_number = effective_number_of_parameter(train_x, X_A, train_y, train_weight, this->beta, beta_A, this->coef0);
+    this->effective_number = this->effective_number_of_parameter(train_x, X_A, train_y, train_weight, this->beta, beta_A, this->coef0);
     this->group_df = A_ind.size();
 
     return;
@@ -344,7 +289,6 @@ public:
   void get_A(T4 &X, T1 &y, Eigen::VectorXi &A, Eigen::VectorXi &I, int &C_max, T2 &beta, T3 &coef0, Eigen::VectorXd &bd, int T0, Eigen::VectorXd &weights,
              Eigen::VectorXi &g_index, Eigen::VectorXi &g_size, int N, double tau, double &train_loss)
   {
-
     Eigen::VectorXi U(this->U_size);
     Eigen::VectorXi U_ind;
     Eigen::VectorXi g_index_U(this->U_size);
@@ -364,7 +308,7 @@ public:
       U = max_k(bd, this->U_size, true);
     }
 
-    int p = X.cols();
+    // int p = X.cols();
     int n = X.rows();
     int C = C_max;
     int iter = 0;
@@ -376,7 +320,7 @@ public:
       {
         delete X_U;
         X_U = &X;
-        U_ind = Eigen::VectorXi::LinSpaced(p, 0, p - 1);
+        U_ind = Eigen::VectorXi::LinSpaced((this->beta).rows(), 0, (this->beta).rows() - 1);
         beta_U = beta;
         g_size_U = g_size;
         g_index_U = g_index;
@@ -386,8 +330,8 @@ public:
       }
       else
       {
-        U_ind = find_ind(U, g_index, g_size, p, N);
-        *X_U = X_seg(X, n, U_ind);
+        U_ind = find_ind(U, g_index, g_size, (this->beta).rows(), N);
+        *X_U = X_seg(X, n, U_ind, this->model_type);
         slice(beta, U_ind, beta_U);
 
         int pos = 0;
@@ -424,7 +368,7 @@ public:
         num++;
 
         Eigen::VectorXi A_ind = find_ind(A_U, g_index_U, g_size_U, U_ind.size(), this->U_size);
-        T4 X_A = X_seg(*X_U, n, A_ind);
+        T4 X_A = X_seg(*X_U, n, A_ind, this->model_type);
         T2 beta_A;
         slice(beta_U, A_ind, beta_A);
 
@@ -464,17 +408,16 @@ public:
           A(tempA++) = i;
 
       // bd in full set
-      Eigen::VectorXi A_ind0 = find_ind(A, g_index, g_size, p, N);
-      T4 X_A0 = X_seg(X, n, A_ind0);
+      Eigen::VectorXi A_ind0 = find_ind(A, g_index, g_size, (this->beta).rows(), N);
+      T4 X_A0 = X_seg(X, n, A_ind0, this->model_type);
       T2 beta_A0;
       slice(beta, A_ind0, beta_A0);
       Eigen::VectorXi U0 = Eigen::VectorXi::LinSpaced(N, 0, N - 1);
-      Eigen::VectorXi U_ind0 = Eigen::VectorXi::LinSpaced(p, 0, p - 1);
+      Eigen::VectorXi U_ind0 = Eigen::VectorXi::LinSpaced((this->beta).rows(), 0, (this->beta).rows() - 1);
       this->sacrifice(X, X_A0, y, beta, beta_A0, coef0, A, I, weights, g_index, g_size, N, A_ind0, bd, U0, U_ind0, 0);
 
       if (this->U_size == N)
       {
-
         for (int i = 0; i < this->always_select.size(); i++)
           bd(this->always_select(i)) = DBL_MAX;
 
@@ -508,7 +451,7 @@ public:
       return false;
 
     // init
-    int p = X.cols();
+    // int p = X.cols();
     int n = X.rows();
 
     int A_size = A.size();
@@ -531,6 +474,10 @@ public:
     Eigen::VectorXi s1 = vector_slice(A, A_min_k);
     Eigen::VectorXi s2 = vector_slice(I, I_max_k);
 
+    // for (int i=0;i<C_max;i++){
+    //   cout<<"try: ("<<s1(i)<<","<<bd(s1(i))<<") -> ("<<s2(i)<<","<<bd(s2(i))<<")"<<endl;///
+    // }
+
     Eigen::VectorXi A_exchange(A_size);
     Eigen::VectorXi A_ind_exchage;
     T4 X_A_exchage;
@@ -541,14 +488,14 @@ public:
     for (int k = C_max; k >= 1;)
     {
       A_exchange = diff_union(A, s1, s2);
-      A_ind_exchage = find_ind(A_exchange, g_index, g_size, p, N);
-      X_A_exchage = X_seg(X, n, A_ind_exchage);
+      A_ind_exchage = find_ind(A_exchange, g_index, g_size, (this->beta).rows(), N);
+      X_A_exchage = X_seg(X, n, A_ind_exchage, this->model_type);
       slice(beta, A_ind_exchage, beta_A_exchange);
       coef0_A_exchange = coef0;
 
-      bool success = primary_model_fit(X_A_exchage, y, weights, beta_A_exchange, coef0_A_exchange, train_loss, A_exchange, g_index, g_size);
+      bool success = this->primary_model_fit(X_A_exchage, y, weights, beta_A_exchange, coef0_A_exchange, train_loss, A_exchange, g_index, g_size);
       // if (success){
-      L = neg_loglik_loss(X_A_exchage, y, weights, beta_A_exchange, coef0_A_exchange, A_exchange, g_index, g_size);
+      L = this->loss_function(X_A_exchage, y, weights, beta_A_exchange, coef0_A_exchange, A_exchange, g_index, g_size, this->lambda_level);
       // }else{
       //   L = train_loss + 1;
       // }
@@ -578,7 +525,10 @@ public:
     return false;
   };
 
-  Eigen::VectorXi inital_screening(T4 &X, T1 &y, T2 &beta, T3 &coef0, Eigen::VectorXi &A, Eigen::VectorXi &I, Eigen::VectorXd &bd, Eigen::VectorXd &weights,
+  virtual void inital_setting(T4 &X, T1 &y, Eigen::VectorXd &weights, Eigen::VectorXi &g_index, Eigen::VectorXi &g_size, int &N) {};
+  virtual void clear_setting(){};
+
+  virtual Eigen::VectorXi inital_screening(T4 &X, T1 &y, T2 &beta, T3 &coef0, Eigen::VectorXi &A, Eigen::VectorXi &I, Eigen::VectorXd &bd, Eigen::VectorXd &weights,
                                    Eigen::VectorXi &g_index, Eigen::VectorXi &g_size, int &N)
   {
 
@@ -586,17 +536,17 @@ public:
     {
       // variable initialization
       int n = X.rows();
-      int p = X.cols();
+      int beta_size = this->get_beta_size(X.rows(), X.cols());
       bd = Eigen::VectorXd::Zero(N);
 
       // calculate beta & d & h
-      Eigen::VectorXi A_ind = find_ind(A, g_index, g_size, p, N);
-      T4 X_A = X_seg(X, n, A_ind);
+      Eigen::VectorXi A_ind = find_ind(A, g_index, g_size, beta_size, N);
+      T4 X_A = X_seg(X, n, A_ind, this->model_type);
       T2 beta_A;
       slice(beta, A_ind, beta_A);
 
       Eigen::VectorXi U = Eigen::VectorXi::LinSpaced(N, 0, N - 1);
-      Eigen::VectorXi U_ind = Eigen::VectorXi::LinSpaced(p, 0, p - 1);
+      Eigen::VectorXi U_ind = Eigen::VectorXi::LinSpaced(beta_size, 0, beta_size - 1);
       this->sacrifice(X, X_A, y, beta, beta_A, coef0, A, I, weights, g_index, g_size, N, A_ind, bd, U, U_ind, 0);
       for (int i = 0; i < this->always_select.size(); i++)
       {
@@ -610,13 +560,13 @@ public:
     return A_new;
   }
 
-  virtual double neg_loglik_loss(T4 &X, T1 &y, Eigen::VectorXd &weights, T2 &beta, T3 &coef0, Eigen::VectorXi &A, Eigen::VectorXi &g_index, Eigen::VectorXi &g_size) = 0;
+  virtual double loss_function(T4 &X, T1 &y, Eigen::VectorXd &weights, T2 &beta, T3 &coef0, Eigen::VectorXi &A, Eigen::VectorXi &g_index, Eigen::VectorXi &g_size, double lambda){ return 0; };
 
-  virtual void sacrifice(T4 &X, T4 &XA, T1 &y, T2 &beta, T2 &beta_A, T3 &coef0, Eigen::VectorXi &A, Eigen::VectorXi &I, Eigen::VectorXd &weights, Eigen::VectorXi &g_index, Eigen::VectorXi &g_size, int N, Eigen::VectorXi &A_ind, Eigen::VectorXd &bd, Eigen::VectorXi &U, Eigen::VectorXi &U_ind, int num) = 0;
+  virtual void sacrifice(T4 &X, T4 &XA, T1 &y, T2 &beta, T2 &beta_A, T3 &coef0, Eigen::VectorXi &A, Eigen::VectorXi &I, Eigen::VectorXd &weights, Eigen::VectorXi &g_index, Eigen::VectorXi &g_size, int N, Eigen::VectorXi &A_ind, Eigen::VectorXd &bd, Eigen::VectorXi &U, Eigen::VectorXi &U_ind, int num){ return; };
 
-  virtual bool primary_model_fit(T4 &X, T1 &y, Eigen::VectorXd &weights, T2 &beta, T3 &coef0, double loss0, Eigen::VectorXi &A, Eigen::VectorXi &g_index, Eigen::VectorXi &g_size) = 0;
+  virtual bool primary_model_fit(T4 &X, T1 &y, Eigen::VectorXd &weights, T2 &beta, T3 &coef0, double loss0, Eigen::VectorXi &A, Eigen::VectorXi &g_index, Eigen::VectorXi &g_size){ return true; };
 
-  virtual double effective_number_of_parameter(T4 &X, T4 &XA, T1 &y, Eigen::VectorXd &weights, T2 &beta, T2 &beta_A, T3 &coef0) = 0;
+  virtual double effective_number_of_parameter(T4 &X, T4 &XA, T1 &y, Eigen::VectorXd &weights, T2 &beta, T2 &beta_A, T3 &coef0){ return this->sparsity_level; };
 };
 
 #endif //SRC_ALGORITHM_H
