@@ -7,8 +7,6 @@
 
 #include "Data.h"
 #include "Algorithm.h"
-#include "model_fit.h"
-// #include "path.h"
 #include <vector>
 #include <random>
 #include <algorithm>
@@ -46,9 +44,9 @@ public:
 
   Metric() = default;
 
-  Metric(int ic_type, double ic_coef = 1.0, bool is_cv = false, int Kfold = 5)
+  Metric(int ic_type, double ic_coef = 1.0, int Kfold = 5)
   {
-    this->is_cv = is_cv;
+    this->is_cv = Kfold > 1;
     this->ic_type = ic_type;
     this->Kfold = Kfold;
     this->ic_coef = ic_coef;
@@ -64,13 +62,13 @@ public:
     }
   };
 
-  void set_cv_init_fit_arg(int p, int M)
+  void set_cv_init_fit_arg(int beta_size, int M)
   {
     for (int i = 0; i < this->Kfold; i++)
     {
       T2 beta_init;
       T3 coef0_init;
-      coef_set_zero(p, M, beta_init, coef0_init);
+      coef_set_zero(beta_size, M, beta_init, coef0_init);
       Eigen::VectorXi A_init;
       Eigen::VectorXd bd_init;
 
@@ -114,32 +112,56 @@ public:
   //   this->cv_initial_coef0[k] = coef0;
   // }
 
-  void set_cv_train_test_mask(Data<T1, T2, T3, T4> &data, int n)
+  void set_cv_train_test_mask(Data<T1, T2, T3, T4> &data, int n, Eigen::VectorXi &cv_fold_id)
   {
     Eigen::VectorXi index_list(n);
     std::vector<int> index_vec((unsigned int)n);
+    std::vector<Eigen::VectorXi> group_list((unsigned int)this->Kfold);
     for (int i = 0; i < n; i++)
     {
       index_vec[i] = i;
     }
-    // std::random_device rd;
-    std::mt19937 g(123);
-    std::shuffle(index_vec.begin(), index_vec.end(), g);
 
-    for (int i = 0; i < n; i++)
-    {
-      index_list(i) = index_vec[i];
-    }
+    if (cv_fold_id.size() == 0){
+      // std::random_device rd;
+      std::mt19937 g(123);
+      std::shuffle(index_vec.begin(), index_vec.end(), g);
 
-    Eigen::VectorXd loss_list(this->Kfold);
-    std::vector<Eigen::VectorXi> group_list((unsigned int)this->Kfold);
-    int group_size = int(n / this->Kfold);
-    for (int k = 0; k < (this->Kfold - 1); k++)
-    {
-      group_list[k] = index_list.segment(int(k * group_size), group_size);
+      for (int i = 0; i < n; i++)
+      {
+        index_list(i) = index_vec[i];
+      }
+
+      Eigen::VectorXd loss_list(this->Kfold);
+      int group_size = int(n / this->Kfold);
+      for (int k = 0; k < (this->Kfold - 1); k++)
+      {
+        group_list[k] = index_list.segment(int(k * group_size), group_size);
+      }
+      group_list[this->Kfold - 1] = index_list.segment(int((this->Kfold - 1) * group_size),
+                                                      n - int(int(this->Kfold - 1) * group_size));
+    }else{
+      // given cv_fold_id
+      auto rule = [cv_fold_id](int i, int j) -> bool
+      {
+          return cv_fold_id(i) < cv_fold_id(j);
+      }; 
+      std::sort(index_vec.begin(), index_vec.end(), rule);
+
+      for (int i = 0; i < n; i++)
+      {
+        index_list(i) = index_vec[i];
+      }
+
+      int k = 0, st = 0, ed = 1;
+      while (k < this->Kfold && ed < n){
+        int mask = cv_fold_id(index_list(st));
+        while (ed < n && mask == cv_fold_id(index_list(ed))) ed++;
+
+        group_list[k] = index_list.segment(st, ed - st);
+        st = ed; ed++; k++;
+      }
     }
-    group_list[this->Kfold - 1] = index_list.segment(int((this->Kfold - 1) * group_size),
-                                                     n - int(int(this->Kfold - 1) * group_size));
     for (int k = 0; k < this->Kfold; k++)
     {
       std::sort(group_list[k].data(), group_list[k].data() + group_list[k].size());
@@ -208,11 +230,11 @@ public:
     double loss;
     if (algorithm->model_type == 1 || algorithm->model_type == 5)
     {
-      loss = train_n * log(algorithm->get_train_loss());
+      loss = train_n * log(algorithm->get_train_loss() - algorithm->lambda_level  * algorithm->beta.cwiseAbs2().sum());
     }
     else
     {
-      loss = 2 * algorithm->get_train_loss();
+      loss = 2 * (algorithm->get_train_loss() - algorithm->lambda_level * algorithm->beta.cwiseAbs2().sum());
     }
 
     if (ic_type == 1)
@@ -235,14 +257,14 @@ public:
       return 0;
   };
 
-  double neg_loglik_loss(T4 &train_x, T1 &train_y, Eigen::VectorXd &train_weight, Eigen::VectorXi &g_index, Eigen::VectorXi &g_size, int train_n, int p, int N, Algorithm<T1, T2, T3, T4> *algorithm)
+  double loss_function(T4 &train_x, T1 &train_y, Eigen::VectorXd &train_weight, Eigen::VectorXi &g_index, Eigen::VectorXi &g_size, int train_n, int p, int N, Algorithm<T1, T2, T3, T4> *algorithm)
   {
     Eigen::VectorXi A = algorithm->get_A_out();
     T2 beta = algorithm->get_beta();
     T3 coef0 = algorithm->get_coef0();
 
-    Eigen::VectorXi A_ind = find_ind(A, g_index, g_size, p, N);
-    T4 X_A = X_seg(train_x, train_n, A_ind);
+    Eigen::VectorXi A_ind = find_ind(A, g_index, g_size, beta.rows(), N);
+    T4 X_A = X_seg(train_x, train_n, A_ind, algorithm->model_type);
 
     T2 beta_A;
     slice(beta, A_ind, beta_A);
@@ -252,41 +274,41 @@ public:
     // {
     //   beta_A(k) = beta(A_ind(k));
     // }
-    double L0 = algorithm->neg_loglik_loss(X_A, train_y, train_weight, beta_A, coef0, A, g_index, g_size);
-
-    return L0;
+    return algorithm->loss_function(X_A, train_y, train_weight, beta_A, coef0, A, g_index, g_size, 0.0);
   }
 
   // to do
-  double fit_and_evaluate_in_metric(Algorithm<T1, T2, T3, T4> *algorithm, Data<T1, T2, T3, T4> &data, std::vector<Algorithm<T1, T2, T3, T4> *> algorithm_list, FIT_ARG<T2, T3> &fit_arg)
+  Eigen::VectorXd fit_and_evaluate_in_metric(std::vector<Algorithm<T1, T2, T3, T4> *> algorithm_list, Data<T1, T2, T3, T4> &data, FIT_ARG<T2, T3> &fit_arg)
   {
-    int N = data.g_num;
-    algorithm->update_sparsity_level(fit_arg.support_size);
-    algorithm->update_lambda_level(fit_arg.lambda);
+    Eigen::VectorXd loss_list(this->Kfold);
 
-    algorithm->update_beta_init(fit_arg.beta_init);
-    algorithm->update_bd_init(fit_arg.bd_init);
-    algorithm->update_coef0_init(fit_arg.coef0_init);
-    algorithm->update_A_init(fit_arg.A_init, N);
-
-    algorithm->fit(data.x, data.y, data.weight, data.g_index, data.g_size, data.n, data.p, data.g_num, data.status, algorithm->Sigma);
-
-    if (algorithm->get_warm_start())
+    if (!is_cv)
     {
-      fit_arg.beta_init = algorithm->get_beta();
-      fit_arg.coef0_init = algorithm->get_coef0();
-      fit_arg.bd_init = algorithm->get_bd();
-    }
+      algorithm_list[0]->update_sparsity_level(fit_arg.support_size);
+      algorithm_list[0]->update_lambda_level(fit_arg.lambda);
+      algorithm_list[0]->update_beta_init(fit_arg.beta_init);
+      algorithm_list[0]->update_bd_init(fit_arg.bd_init);
+      algorithm_list[0]->update_coef0_init(fit_arg.coef0_init);
+      algorithm_list[0]->update_A_init(fit_arg.A_init, data.g_num);
 
-    if (is_cv)
+      algorithm_list[0]->fit(data.x, data.y, data.weight, data.g_index, data.g_size, data.n, data.p, data.g_num);
+
+      if (algorithm_list[0]->get_warm_start())
+      {
+        fit_arg.beta_init = algorithm_list[0]->get_beta();
+        fit_arg.coef0_init = algorithm_list[0]->get_coef0();
+        fit_arg.bd_init = algorithm_list[0]->get_bd();
+      }
+
+      loss_list(0) = this->ic(data.n, data.M, data.g_num, algorithm_list[0]);
+    }
+    else
     {
       Eigen::VectorXi g_index = data.g_index;
       Eigen::VectorXi g_size = data.g_size;
       int p = data.p;
       int N = data.g_num;
 
-      Eigen::VectorXd loss_list(this->Kfold);
-
 #pragma omp parallel for
       ///////////////////////parallel/////////////////////////
       for (int k = 0; k < this->Kfold; k++)
@@ -307,41 +329,33 @@ public:
         algorithm_list[k]->update_sparsity_level(fit_arg.support_size);
         algorithm_list[k]->update_lambda_level(fit_arg.lambda);
 
-        if (algorithm_list[k]->get_warm_start())
-        {
-
-          algorithm_list[k]->update_beta_init(this->cv_init_fit_arg[k].beta_init);
-          algorithm_list[k]->update_bd_init(this->cv_init_fit_arg[k].bd_init);
-          algorithm_list[k]->update_coef0_init(this->cv_init_fit_arg[k].coef0_init);
-          algorithm_list[k]->update_A_init(this->cv_init_fit_arg[k].A_init, N);
-          // beta_init = this->cv_initial_model_param.col(k).eval();
-          // algorithm->update_beta_init(beta_init);
-          // algorithm->update_coef0_init(this->cv_initial_coef0[k]);
-          // algorithm->update_A_init(this->cv_initial_A[k], N);
-        }
+        algorithm_list[k]->update_beta_init(this->cv_init_fit_arg[k].beta_init);
+        algorithm_list[k]->update_bd_init(this->cv_init_fit_arg[k].bd_init);
+        algorithm_list[k]->update_coef0_init(this->cv_init_fit_arg[k].coef0_init);
+        algorithm_list[k]->update_A_init(this->cv_init_fit_arg[k].A_init, N);
+        // beta_init = this->cv_initial_model_param.col(k).eval();
+        // algorithm->update_beta_init(beta_init);
+        // algorithm->update_coef0_init(this->cv_initial_coef0[k]);
+        // algorithm->update_A_init(this->cv_initial_A[k], N);
         // algorithm->update_train_mask(this->train_mask_list[k]);
-        /// ??????????????????????????????????????????????????????????????
-        algorithm_list[k]->fit(this->train_X_list[k], this->train_y_list[k], this->train_weight_list[k], g_index, g_size, train_n, p, N, data.status, algorithm_list[k]->Sigma);
+        // ??????????????????????????????????????????????????????????????
+        algorithm_list[k]->fit(this->train_X_list[k], this->train_y_list[k], this->train_weight_list[k], g_index, g_size, train_n, p, N);
 
         if (algorithm_list[k]->get_warm_start())
         {
-          this->cv_init_fit_arg[k].beta_init = algorithm->get_beta();
-          this->cv_init_fit_arg[k].coef0_init = algorithm->get_coef0();
-          this->cv_init_fit_arg[k].bd_init = algorithm->get_bd();
+          this->cv_init_fit_arg[k].beta_init = algorithm_list[k]->get_beta();
+          this->cv_init_fit_arg[k].coef0_init = algorithm_list[k]->get_coef0();
+          this->cv_init_fit_arg[k].bd_init = algorithm_list[k]->get_bd();
           // this->update_cv_initial_model_param(algorithm->get_beta(), k);
           // this->update_cv_initial_A(algorithm->get_A_out(), k);
           // this->update_cv_initial_coef0(algorithm->get_coef0(), k);
         }
 
-        loss_list(k) = this->neg_loglik_loss(this->test_X_list[k], this->test_y_list[k], this->test_weight_list[k], g_index, g_size, test_n, p, N, algorithm_list[k]);
+        loss_list(k) = this->loss_function(this->test_X_list[k], this->test_y_list[k], this->test_weight_list[k], g_index, g_size, test_n, p, N, algorithm_list[k]);
       }
+    }
 
-      return loss_list.mean();
-    }
-    else
-    {
-      return this->ic(data.n, data.M, data.g_num, algorithm);
-    }
+    return loss_list;
   };
 };
 
