@@ -4,40 +4,44 @@ library(testthat)
 
 sparse_batch_check <- function(abess_fit1, abess_fit2) {
   expect_equal(abess_fit1[["tune.value"]], abess_fit2[["tune.value"]],
-               tolerance = 1e-2)
+    tolerance = 1e-2
+  )
   expect_equal(abess_fit1[["dev"]], abess_fit2[["dev"]],
-               tolerance = 1e-2)
+    tolerance = 1e-2
+  )
   if (abess_fit1[["family"]] %in% c("mgaussian", "multinomial")) {
     support_size_num <- length(abess_fit1[["beta"]])
     for (i in 1:support_size_num) {
       if (length(abess_fit1[["beta"]][[1]]@i) != 0) {
         expect_lt(mean(abess_fit1[["beta"]][[i]]@i - abess_fit2[["beta"]][[i]]@i), 1e-3)
         expect_equal(abess_fit1[["beta"]][[i]]@x, abess_fit2[["beta"]][[i]]@x,
-                     tolerance = 1e-2)
+          tolerance = 1e-2
+        )
       }
     }
   } else {
     expect_lt(mean(abess_fit1[["beta"]]@i - abess_fit2[["beta"]]@i), 1e-3)
     expect_equal(abess_fit1[["beta"]]@x, abess_fit2[["beta"]]@x,
-                 tolerance = 1e-2)
+      tolerance = 1e-2
+    )
   }
 }
 
 test_that("Sparse matrix (gaussian) works", {
-  n <- 100
+  n <- 30
   p <- 20
   support.size <- 3
   dataset <- generate.data(n, p, support.size)
   abess_fit1 <- abess(dataset[["x"]], dataset[["y"]])
   abess_fit2 <- abess(as.matrix(dataset[["x"]]), dataset[["y"]])
-  
+
   abess_fit1[["call"]] <- NULL
   abess_fit2[["call"]] <- NULL
   expect_true(all.equal(abess_fit1, abess_fit2))
 })
 
 test_that("Sparse matrix (logistic) works", {
-  n <- 100
+  n <- 200
   p <- 20
   support.size <- 3
   dataset <- generate.data(n, p, support.size, family = "binomial")
@@ -47,7 +51,7 @@ test_that("Sparse matrix (logistic) works", {
     abess(dataset[["x"]], dataset[["y"]], family = "binomial")
   abess_fit2 <-
     abess(as.matrix(dataset[["x"]]), dataset[["y"]], family = "binomial")
-  
+
   abess_fit1[["call"]] <- NULL
   abess_fit2[["call"]] <- NULL
   sparse_batch_check(abess_fit1, abess_fit2)
@@ -65,7 +69,7 @@ test_that("Sparse matrix (poisson) works", {
     abess(dataset[["x"]], dataset[["y"]], family = "poisson")
   abess_fit2 <-
     abess(as.matrix(dataset[["x"]]), dataset[["y"]], family = "poisson")
-  
+
   abess_fit1[["call"]] <- NULL
   abess_fit2[["call"]] <- NULL
   sparse_batch_check(abess_fit1, abess_fit2)
@@ -82,7 +86,7 @@ test_that("Sparse matrix (cox) works", {
     abess(dataset[["x"]], dataset[["y"]], family = "poisson")
   abess_fit2 <-
     abess(as.matrix(dataset[["x"]]), dataset[["y"]], family = "poisson")
-  
+
   abess_fit1[["call"]] <- NULL
   abess_fit2[["call"]] <- NULL
   sparse_batch_check(abess_fit1, abess_fit2)
@@ -99,7 +103,7 @@ test_that("Sparse matrix (mgaussian) works", {
     abess(dataset[["x"]], dataset[["y"]], family = "mgaussian")
   abess_fit2 <-
     abess(as.matrix(dataset[["x"]]), dataset[["y"]], family = "mgaussian")
-  
+
   abess_fit1[["call"]] <- NULL
   abess_fit2[["call"]] <- NULL
   sparse_batch_check(abess_fit1, abess_fit2)
@@ -107,7 +111,7 @@ test_that("Sparse matrix (mgaussian) works", {
 
 
 test_that("Sparse matrix (multinomial) works", {
-  n <- 100
+  n <- 200
   p <- 20
   support.size <- 3
   dataset <-
@@ -118,7 +122,7 @@ test_that("Sparse matrix (multinomial) works", {
     abess(dataset[["x"]], dataset[["y"]], family = "multinomial")
   abess_fit2 <-
     abess(as.matrix(dataset[["x"]]), dataset[["y"]], family = "multinomial")
-  
+
   abess_fit1[["call"]] <- NULL
   abess_fit2[["call"]] <- NULL
   sparse_batch_check(abess_fit1, abess_fit2)
